package datavisualizer;

import java.io.Serializable;

/**
 * Represents one (x, y, z) data item for an xyz-series.  Originally from
 * JFreeChart - I modified it.
 *  Original author - David Gilbert
 * <p>Title: Data Visualizer</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class XYZDataItem
    implements Serializable {
  /** The x-value. */
  private Number x;

  /** The y-value. */
  private Number y;

  /** The z-value. */
  private Number z;

  /**
   * Constructs a new data pair.
   *
   * @param x  the x-value.
   * @param y  the y-value.
   * @param z  the z-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public XYZDataItem(Number x, Number y, Number z) {
    this.x = x;
    this.y = y;
    this.z = z;
  }

  /**
   * Constructs a new data pair.
   *
   * @param x  the x-value.
   * @param y  the y-value.
   * @param z  the z-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public XYZDataItem(double x, double y, double z) {
    this(new Double(x), new Double(y), new Double(z));
  }

  /**
   * Returns the x-value.
   *
   * @return the x-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getX() {
    return this.x;
  }

  /**
   * Returns the y-value.
   *
   * @return the y-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getY() {
    return this.y;
  }

  /**
   * Returns the z-value.
   *
   * @return the z-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getZ() {
    return this.z;
  }

  /**
   * Sets the y-value for this data pair.
   * <P>
   * Note that there is no corresponding method to change the x-value.
   *
   * @param y  the new y-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void setY(Number y) {
    this.y = y;
  }

  /**
   * Returns an integer indicating the order of this data pair object relative to another object.
   * <P>
   * For the order we consider only the x-value:
   * negative == "less-than", zero == "equal", positive == "greater-than".
   *
   * @param o1  the object being compared to.
   *
   * @return  an integer indicating the order of this data pair object
   *      relative to another object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  /*  public int compareTo(Object o1) {
        int result;
        // CASE 1 : Comparing to another TimeSeriesDataPair object
        // -------------------------------------------------------
        if (o1 instanceof XYZDataItem) {
            XYZDataItem dataItem = (XYZDataItem) o1;
       double compare = this.x.doubleValue() - dataItem.getX().doubleValue();
            if (compare > 0) {
                result = 1;
            }
            else {
                if (compare < 0) {
                    result = -1;
                }
                else {
                    result = 0;
                }
            }
        }
        // CASE 2 : Comparing to a general object
        // ---------------------------------------------
        else {
            // consider time periods to be ordered after general objects
            result = 1;
        }
        return result;
    } */

  /**
   * Returns a clone of this XYDataPair.
   *
   * @return a clone of the data pair.
   */
  /*  public Object clone() {
        Object clone = null;
        try {
            clone = super.clone();
        }
        catch (CloneNotSupportedException e) { // won't get here...
            System.err.println("XYDataPair.clone(): operation not supported.");
        }
        return clone;
    } */

}