package javawrapper;

import java.util.Vector;

/**
 * Base class for detailed output settings.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class DetailedOutputSettings {

  protected Vector mp_oFloats, /**<Vector of DataMembers representing floats*/
      mp_oInts, /**<Vector of DataMembers representing ints*/
      mp_oChars, /**<Vector of DataMembers representing chars*/
      mp_oBools; /**<Vector of DataMembers representing bools*/
  protected int m_iSaveFreq; /**<Save frequency, in timesteps*/

  /**
   * Constructor.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DetailedOutputSettings() {
    mp_oFloats = new Vector(0);
    mp_oInts = new Vector(0);
    mp_oBools = new Vector(0);
    mp_oChars = new Vector(0);
    m_iSaveFreq = 0;
  }

  /**
   * Gets the save frequency.
   * @return Save frequency, in timesteps.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetSaveFrequency() {
    return m_iSaveFreq;
  }

  /**
   * Sets the save frequency.
   * @param i Save frequency, in timesteps.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetSaveFrequency(int i) {
    m_iSaveFreq = i;
  }

  /**
   * Add a new float data label.  If it already exists it won't be added again.
   * @param sLabel Float data label.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddFloat(String sLabel) {
    DataMember oDataMember;
    int i;
    for (i = 0; i < mp_oFloats.size(); i++) {
      oDataMember = (DataMember) mp_oFloats.elementAt(i);
      if (sLabel.equals(oDataMember.GetDisplayName())) {
        return;
      }
    }
    mp_oFloats.add(new javawrapper.DataMember(sLabel));
  }

  /**
   * Add a new int data label.  If it already exists it won't be added again.
   * @param sLabel Int data label.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddInt(String sLabel) {
    DataMember oDataMember;
    int i;
    for (i = 0; i < mp_oInts.size(); i++) {
      oDataMember = (DataMember) mp_oInts.elementAt(i);
      if (sLabel.equals(oDataMember.GetDisplayName())) {
        return;
      }
    }
    mp_oInts.add(new javawrapper.DataMember(sLabel));
  }

  /**
   * Add a new char data label.  If it already exists it won't be added again.
   * @param sLabel Char data label.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddChar(String sLabel) {
    DataMember oDataMember;
    int i;
    for (i = 0; i < mp_oChars.size(); i++) {
      oDataMember = (DataMember) mp_oChars.elementAt(i);
      if (sLabel.equals(oDataMember.GetDisplayName())) {
        return;
      }
    }
    mp_oChars.add(new javawrapper.DataMember(sLabel));
  }

  /**
   * Add a new bool data label.  If it already exists it won't be added again.
   * @param sLabel bool data label.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddBool(String sLabel) {
    DataMember oDataMember;
    int i;
    for (i = 0; i < mp_oBools.size(); i++) {
      oDataMember = (DataMember) mp_oBools.elementAt(i);
      if (sLabel.equals(oDataMember.GetDisplayName())) {
        return;
      }
    }
    mp_oBools.add(new javawrapper.DataMember(sLabel));
  }

  /**
   * Get the number of float data members.
   * @return Number of data members.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfFloats() {
    return mp_oFloats.size();
  }

  /**
   * Get the number of int data members.
   * @return Number of data members.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfInts() {
    return mp_oInts.size();
  }

  /**
   * Get the number of char data members.
   * @return Number of data members.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfChars() {
    return mp_oChars.size();
  }

  /**
   * Get the number of bool data members.
   * @return Number of data members.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfBools() {
    return mp_oBools.size();
  }

  /**
   * Removes an integer data member.
   * @param i int Index of data member to remove.
   */
  public void RemoveInt(int i){
    if (i < 0 || i > mp_oInts.size() - 1) {
      return;
    }
    mp_oInts.remove(i);
  }

  /**
   * Removes a float data member.
   * @param i int Index of data member to remove.
   */
  public void RemoveFloat(int i){
    if (i < 0 || i > mp_oFloats.size() - 1) {
      return;
    }
    mp_oFloats.remove(i);
  }

  /**
   * Removes a boolean data member.
   * @param i int Index of data member to remove.
   */
  public void RemoveBool(int i){
    if (i < 0 || i > mp_oBools.size() - 1) {
      return;
    }
    mp_oBools.remove(i);
  }

  /**
   * Removes a char data member.
   * @param i int Index of data member to remove.
   */
  public void RemoveChar(int i){
    if (i < 0 || i > mp_oChars.size() - 1) {
      return;
    }
    mp_oChars.remove(i);
  }


  /**
   * Gets a float data member
   * @param iIndex of the float data member
   * @return The data member
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DataMember GetFloat(int iIndex) {
    if (iIndex >= mp_oFloats.size() || iIndex < 0) {
      return null;
    }

    return (DataMember) mp_oFloats.elementAt(iIndex);
  }

  /**
   * Get the label of an int data member
   * @param iIndex of the int data member
   * @return The data member
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DataMember GetInt(int iIndex) {
    if (iIndex >= mp_oInts.size() || iIndex < 0) {
      return null;
    }

    return (DataMember) mp_oInts.elementAt(iIndex);
  }

  /**
   * Get the label of a char data member
   * @param iIndex of the char data member
   * @return The data member
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DataMember GetChar(int iIndex) {
    if (iIndex >= mp_oChars.size() || iIndex < 0) {
      return null;
    }

    return (DataMember) mp_oChars.elementAt(iIndex);
  }

  /**
   * Gets a bool data member
   * @param iIndex of the bool data member
   * @return The data member
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DataMember GetBool(int iIndex) {
    if (iIndex >= mp_oBools.size() || iIndex < 0) {
      return null;
    }

    return (DataMember) mp_oBools.elementAt(iIndex);
  }

  /**
   * Copies data members from one DetailedOutputSettings object to another.  This
   * can be used to create deep clones.
   * @param oCopyTarget Object to which to copy settings
   * @param oCopySource Object from which to copy settings.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public static void CopyData(DetailedOutputSettings oCopyTarget,
                              DetailedOutputSettings oCopySource) {
    if (oCopyTarget == null || oCopySource == null) {
      return;
    }
    int j;

    DataMember oMember;
    oCopyTarget.SetSaveFrequency(oCopySource.GetSaveFrequency());

    oCopyTarget.mp_oFloats.clear();
    oCopyTarget.mp_oBools.clear();
    oCopyTarget.mp_oInts.clear();
    oCopyTarget.mp_oChars.clear();

    for (j = 0; j < oCopySource.mp_oFloats.size(); j++) {
      oMember = (DataMember) oCopySource.mp_oFloats.elementAt(j);
      oCopyTarget.mp_oFloats.add( (DataMember) oMember.clone());
    }
    for (j = 0; j < oCopySource.mp_oInts.size(); j++) {
      oMember = (DataMember) oCopySource.mp_oInts.elementAt(j);
      oCopyTarget.mp_oInts.add( (DataMember) oMember.clone());
    }
    for (j = 0; j < oCopySource.mp_oChars.size(); j++) {
      oMember = (DataMember) oCopySource.mp_oChars.elementAt(j);
      oCopyTarget.mp_oChars.add( (DataMember) oMember.clone());
    }
    for (j = 0; j < oCopySource.mp_oBools.size(); j++) {
      oMember = (DataMember) oCopySource.mp_oBools.elementAt(j);
      oCopyTarget.mp_oBools.add( (DataMember) oMember.clone());
    }
  }
}
