package javawrapper;

/**
 * Model exception class.  This is what will be thrown by application
 * functions.  Other types of exceptions are wrapped in this class.  This has
 * the same structure as the C++ error structure.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class ModelException
    extends Throwable {

  /**
   * Constructor.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public ModelException() {
    iErrorCode = -1;
    sCallingFunction = "";
    sMessage = "";
  }

  /**
   * Constructor.
   * @param iErrorCode Error code for the exception.
   * @param sCallingFunction The function that threw the error.
   * @param sMessage Error message.
   */
  public ModelException(int iErrorCode, String sCallingFunction, String sMessage) {
    this.iErrorCode = iErrorCode;
    this.sCallingFunction = sCallingFunction;
    this.sMessage = sMessage;
  }

  public int iErrorCode; /**<Error code.  Find a list in ErrorGUI.*/
  public String sCallingFunction; /**<The function that threw the error.*/
  public String sMessage; /**<Error message.*/
}