package javawrapper;

/**
 * This is the organizer class for the dead remover behavor.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class RemoveDeadBehaviors
    extends BehaviorTypeBase {

  /**
   * Constructor
   * @param oManager GUIManager object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public RemoveDeadBehaviors(GUIManager oManager) {
    super(oManager, "Dead tree remover", "");

    //Set up behavior list
    mp_oChildBehaviors = new Behavior[1];

    //Add a behavior for dead removal
    mp_oChildBehaviors[0] = new Behavior("removedead", "Dead tree remover",
                                         "removedead", (float)2, (float)1);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.STUMP, true);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SNAG, true);
  }

  /**
   * Overload this to do nothing.
   * @param jOut Not used.
   * @param oPop Not used.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void WriteXML(java.io.BufferedWriter jOut, TreePopulation oPop) {
    ;
  };

  /**
   * No setup required.  Does nothing.
   * @param oPop Not used.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void DoSetup(TreePopulation oPop) {
    ;
  };

  /**
  * Makes sure that there is a mortality behavior enabled if this behavior
  * is enabled.
  * @param oPop Not used.
  * @throws ModelException if the dead tree remover is enabled but no
  * mortality behavior is.
  *
  * <br>Edit history:
  * <br>------------------
  * <br>April 28, 2004: Submitted in beta version (LEM)
  * <br>December 10, 2004:  Added the described above functionality (LEM)
  */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    if (mp_oChildBehaviors[0].m_bIsEnabled) {
      if (m_oManager.GetMortalityBehaviors().AnyBehaviorsEnabled() == false) {
        throw (new ModelException(ErrorGUI.DATA_MISSING, "JAVA",
                                  "The " + mp_oChildBehaviors[0].GetDescriptor() +
                                  " behavior must be used " +
                                  "with a mortality behavior."));
      }
    }
  }
}
