package javawrapper;

import junit.framework.*;

/**
 * Tests the ModelEnum class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestModelEnum
    extends TestCase {

  public TestModelEnum(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Tests the ModelEnum equality method
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void testEquals() {
    try {
      //The object to test against
      ModelEnum oToTest = new ModelEnum(new int[] {1, 2, 3}
                                        , new String[] {"test 1", "test 2",
                                        "test 3"}
                                        , "test descriptor", "test xml tag");
      oToTest.SetValue(2);

      assertFalse(oToTest.equals(null)); //null
      assertFalse(oToTest.equals(new Float(1))); //non-ModelEnum

      //ModelEnum with different value
      ModelEnum oToTest2 = new ModelEnum(new int[] {1, 2, 3}
                                         , new String[] {"test 1", "test 2",
                                         "test 3"}
                                         , "test descriptor", "test xml tag");
      oToTest2.SetValue(1);
      assertFalse(oToTest.equals(oToTest2));
      //Change the value to make it work
      oToTest2.SetValue(2);
      assertTrue(oToTest.equals(oToTest2));

      //ModelEnum with different XML tag
      oToTest2 = null;
      oToTest2 = new ModelEnum(new int[] {1, 2, 3}
                               , new String[] {"test 1", "test 2", "test 3"}
                               , "test descriptor", "test xml tag 1");
      oToTest2.SetValue(2);
      assertFalse(oToTest.equals(oToTest2));

      //ModelEnum with different descriptor
      oToTest2 = null;
      oToTest2 = new ModelEnum(new int[] {1, 2, 3}
                               , new String[] {"test 1", "test 2", "test 3"}
                               , "test descriptor 1", "test xml tag");
      oToTest2.SetValue(2);
      assertFalse(oToTest.equals(oToTest2));

      //ModelEnum with null allowed values
      oToTest2 = null;
      oToTest2 = new ModelEnum(null, new String[] {"test 1", "test 2", "test 3"}
                               , "test descriptor", "test xml tag");
      assertFalse(oToTest.equals(oToTest2));

      //ModelEnum with null value labels
      oToTest2 = null;
      oToTest2 = new ModelEnum(new int[] {1, 2, 3}
                               , null, "test descriptor", "test xml tag");
      assertFalse(oToTest.equals(oToTest2));

      //ModelEnum with different allowed values
      oToTest2 = null;
      oToTest2 = new ModelEnum(new int[] {0, 2, 3}
                               , new String[] {"test 1", "test 2", "test 3"}
                               , "test descriptor", "test xml tag");
      oToTest2.SetValue(2);
      assertFalse(oToTest.equals(oToTest2));

      //ModelEnum with different value labels
      oToTest2 = null;
      oToTest2 = new ModelEnum(new int[] {1, 2, 3}
                               , new String[] {"test 1", "test 2 diff",
                               "test 3"}
                               , "test descriptor", "test xml tag");
      oToTest2.SetValue(2);
      assertFalse(oToTest.equals(oToTest2));

      System.out.println("ModelEnum test succeeded.");
    }
    catch (ModelException oErr) {
      fail("ModelEnum equality test failed with message " + oErr.sMessage);
    }
  }

}
