package javawrapper;

import junit.framework.*;
import javax.swing.JDialog;

/**
 * Tests the RundataGridSetup class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestRundataGridSetup
    extends TestCase {

  public TestRundataGridSetup(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Tests the case where there are no settings in the OutputBehaviors object
   * to load when the window is created
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void testLoadNoSettings() {
    try {
      OutputBehaviors oParent = new OutputBehaviors(null);
      JDialog jFake = new JDialog();
      DetailedOutputGridSetup oTester = new DetailedOutputGridSetup(jFake, oParent);

      //Verify that there are no choices in the grid
      assertEquals(1, oTester.m_jGridListCombo.getItemCount());
      String sChoice = (String) oTester.m_jGridListCombo.getItemAt(0);
      assertTrue(sChoice.startsWith("---"));

      //Verify that there is nothing in data members
      assertEquals(0, oTester.jDataMemberListModel.getSize());

      //Verify that there is nothing in the "what saved" box
      assertEquals(0, oTester.jSaveListModel.getSize());
    }
    catch (ModelException oErr) {
      fail("TestLoadNoSettings failed with message " + oErr.sMessage);
    }
  }

  /**
   * Tests the case where there are settings in the OutputBehaviors object
   * to load when the window is created
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void testLoadSettings() {
    try {

      OutputBehaviors oParent = new OutputBehaviors(null);
      JDialog jFake = new JDialog();

      //Create some settings to load
      DetailedGridSettings oSettings = new DetailedGridSettings("Test Grid 1");
      oSettings.AddChar("Char1");
      oSettings.AddInt("Integer 1");
      oSettings.AddInt("Integer 2");
      oSettings.SetSaveFrequency(2);
      oParent.AddDetailedGridSettings(oSettings);

      oSettings = null;
      oSettings = new DetailedGridSettings("Test Grid 2");
      oSettings.AddBool("Bool1");
      oSettings.AddChar("Char 1");
      oSettings.SetSaveFrequency(2);
      oParent.AddDetailedGridSettings(oSettings);

      DetailedOutputGridSetup oTester = new DetailedOutputGridSetup(jFake, oParent);

      //Verify that there are 2 choices in the grid
      /*    assertEquals(3, oTester.jGridListCombo.getItemCount());
          String sChoice = (String) oTester.jGridListCombo.getItemAt(0);
          assertTrue(sChoice.startsWith("---"));
          sChoice = (String) oTester.jGridListCombo.getItemAt(1);
          assertTrue(sChoice.equals("Test Grid 1"));
          sChoice = (String) oTester.jGridListCombo.getItemAt(2);
          assertTrue(sChoice.equals("Test Grid 2"));
          //Verify that there is nothing in data members
          assertEquals(0, oTester.jDataMemberListModel.getSize());
          //Choose the no-grid line and verify that there's nothing in data members
          oTester.jGridListCombo.setSelectedIndex(0);
          assertEquals(0, oTester.jDataMemberListModel.getSize());
          //Choose the first grid and verify the number of data members
          oTester.jGridListCombo.setSelectedIndex(1);
          assertEquals(3, oTester.jDataMemberListModel.getSize());
          //Choose the second grid and verify the number of data members
          oTester.jGridListCombo.setSelectedIndex(2);
          assertEquals(2, oTester.jDataMemberListModel.getSize()); */

      //Verify that there are two lines in the "what saved" box
      assertEquals(2, oTester.jSaveListModel.getSize());
    }
    catch (ModelException oErr) {
      fail("TestLoadSettings failed with message " + oErr.sMessage);
    }
  }
}
