package javawrapper;

import junit.framework.*;
import java.io.FileWriter;
import java.io.File;

/**
 * Tests the SubstrateBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>June 3, 2005: Created (LEM)
 */
public class TestSubstrateBehaviors
    extends TestCase {

  public TestSubstrateBehaviors(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Tests to make sure old substrate files are properly read.
   */
  public void testBackwardsCompatibility() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Load an old file
      oManager.ClearCurrentData();
      sFileName = WriteXMLOldFile1();
      oManager.InputXMLParameterFile(sFileName);
      SubstrateBehaviors oSubs = oManager.GetSubstrateBehaviors();

      //Verify that all the values are set correctly
      assertEquals(oSubs.m_fScarSoilA.GetValue(), -5.1E-4, 0.001);
      assertEquals(oSubs.m_fScarSoilB.GetValue(), 4.4, 0.001);
      assertEquals(oSubs.m_fTipUpA.GetValue(), -7.0E-4, 0.001);
      assertEquals(oSubs.m_fTipUpB.GetValue(), 4.3, 0.001);
      assertEquals(oSubs.m_fFreshlogA.GetValue(), -0.05, 0.001);
      assertEquals(oSubs.m_fFreshlogB.GetValue(), 6.5, 0.001);
      assertEquals(oSubs.m_fDecayedlogA.GetValue() , -0.7985, 0.001);
      assertEquals(oSubs.m_fDecayedlogB.GetValue(), 1, 0.001);
      assertEquals(oSubs.m_fInitCondScarSoil.GetValue(), 0.18, 0.001);
      assertEquals(oSubs.m_fPartialCutScarSoil.GetValue(), 0.17,
                   0.001);
      assertEquals(oSubs.m_fGapCutScarSoil.GetValue(), 0.45, 0.001);
      assertEquals(oSubs.m_fClearCutScarSoil.GetValue(), 0.36, 0.001);
      assertEquals(oSubs.m_fInitCondTipup.GetValue(), 0.3, 0.001);
      assertEquals(oSubs.m_fPartialCutTipup.GetValue(), 0.15, 0.001);
      assertEquals(oSubs.m_fGapCutTipup.GetValue(), 0.34, 0.001);
      assertEquals(oSubs.m_fClearCutTipup.GetValue(), 0.35, 0.001);
      assertEquals(oSubs.m_fInitCondFreshLog.GetValue(), 0.01, 0.001);
      assertEquals(oSubs.m_fPartialCutFreshLog.GetValue(), 0.06,
                   0.001);
      assertEquals(oSubs.m_fGapCutFreshLog.GetValue(), 0.09, 0.001);
      assertEquals(oSubs.m_fClearCutFreshLog.GetValue(), 0.16, 0.001);
      assertEquals(oSubs.m_fInitCondDecLog.GetValue(), 0.11, 0.001);
      assertEquals(oSubs.m_fPartialCutDecLog.GetValue(), 0.04, 0.001);
      assertEquals(oSubs.m_fGapCutDecLog.GetValue(), 0.12, 0.001);
      assertEquals(oSubs.m_fClearCutDecLog.GetValue(), 0.01, 0.001);
      assertEquals(oSubs.m_fRootTipupFactor.GetValue(), 3.1, 0.001);
      assertEquals(oSubs.m_fMossProportion.GetValue(), 0, 0.001);
      assertEquals(oSubs.m_iDirectionalTreeFall.GetValue(), 0);
      assertEquals(oSubs.m_iMaxDecayTime.GetValue(), 50);
      assertEquals( ( (Float) oSubs.mp_fProportionOfDeadThatFall.GetValue().
                     elementAt(0)).floatValue(), 0.7, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfDeadThatFall.GetValue().
                     elementAt(1)).floatValue(), 0.6, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().
                     elementAt(0)).floatValue(), 0.5, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().
                     elementAt(1)).floatValue(), 0.4, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().
                     elementAt(0)).floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().
                     elementAt(1)).floatValue(), 0.2, 0.001);

     //Load an old file
      oManager.ClearCurrentData();
      sFileName = WriteXMLOldFile2();
      oManager.InputXMLParameterFile(sFileName);
      oSubs = oManager.GetSubstrateBehaviors();

      //Verify that all the values are set correctly
      assertEquals(oSubs.m_fScarSoilA.GetValue(), -5.1, 0.001);
      assertEquals(oSubs.m_fScarSoilB.GetValue(), 4.4, 0.001);
      assertEquals(oSubs.m_fTipUpA.GetValue(), -7.0, 0.001);
      assertEquals(oSubs.m_fTipUpB.GetValue(), 4.3, 0.001);
      assertEquals(oSubs.m_fFreshlogA.GetValue(), -0.05, 0.001);
      assertEquals(oSubs.m_fFreshlogB.GetValue(), 6.5, 0.001);
      assertEquals(oSubs.m_fDecayedlogA.GetValue(), -1.38629, 0.001);
      assertEquals(oSubs.m_fDecayedlogB.GetValue(), 1, 0.001);
      assertEquals(oSubs.m_fInitCondScarSoil.GetValue(), 0.23, 0.001);
      assertEquals(oSubs.m_fPartialCutScarSoil.GetValue(), 0.17, 0.001);
      assertEquals(oSubs.m_fGapCutScarSoil.GetValue(), 0.45, 0.001);
      assertEquals(oSubs.m_fClearCutScarSoil.GetValue(), 0.36, 0.001);
      assertEquals(oSubs.m_fInitCondTipup.GetValue(), 0.2, 0.001);
      assertEquals(oSubs.m_fPartialCutTipup.GetValue(), 0.15, 0.001);
      assertEquals(oSubs.m_fGapCutTipup.GetValue(), 0.34, 0.001);
      assertEquals(oSubs.m_fClearCutTipup.GetValue(), 0.32, 0.001);
      assertEquals(oSubs.m_fInitCondFreshLog.GetValue(), 0.01, 0.001);
      assertEquals(oSubs.m_fPartialCutFreshLog.GetValue(), 0.06, 0.001);
      assertEquals(oSubs.m_fGapCutFreshLog.GetValue(), 0.09, 0.001);
      assertEquals(oSubs.m_fClearCutFreshLog.GetValue(), 0.13, 0.001);
      assertEquals(oSubs.m_fInitCondDecLog.GetValue(), 0.11, 0.001);
      assertEquals(oSubs.m_fPartialCutDecLog.GetValue(), 0.04, 0.001);
      assertEquals(oSubs.m_fGapCutDecLog.GetValue(), 0.12, 0.001);
      assertEquals(oSubs.m_fClearCutDecLog.GetValue(), 0.02, 0.001);
      assertEquals(oSubs.m_fRootTipupFactor.GetValue(), 3.1, 0.001);
      assertEquals(oSubs.m_fMossProportion.GetValue(), 0.7, 0.001);
      assertEquals(oSubs.m_iDirectionalTreeFall.GetValue(), 1);
      assertEquals(oSubs.m_iMaxDecayTime.GetValue(), 3);
      assertEquals( ( (Float) oSubs.mp_fProportionOfDeadThatFall.GetValue().elementAt(0)).floatValue(), 1, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfDeadThatFall.GetValue().elementAt(1)).floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().elementAt(1)).floatValue(), 0.18, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().elementAt(0)).floatValue(), 0.4, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().elementAt(1)).floatValue(), 0.8, 0.001);
    }
    catch (ModelException oErr) {
      fail("Substrate backwards compatibility failed with message " +
           oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      if (sFileName != null) {
        new File(sFileName).delete();
      }
    }
  }

  /**
   * Tests to make sure old substrate files are properly read.
   */
  public void testParFileRead() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Load a file
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      SubstrateBehaviors oSubs = oManager.GetSubstrateBehaviors();

      //Verify that all the values are set correctly
      assertEquals(oSubs.m_fScarSoilA.GetValue(), -5.1, 0.001);
      assertEquals(oSubs.m_fScarSoilB.GetValue(), 4.4, 0.001);
      assertEquals(oSubs.m_fTipUpA.GetValue(), -7.0, 0.001);
      assertEquals(oSubs.m_fTipUpB.GetValue(), 4.3, 0.001);
      assertEquals(oSubs.m_fFreshlogA.GetValue(), -0.05, 0.001);
      assertEquals(oSubs.m_fFreshlogB.GetValue(), 6.5, 0.001);
      assertEquals(oSubs.m_fDecayedlogA.GetValue(), -0.2, 0.001);
      assertEquals(oSubs.m_fDecayedlogB.GetValue(), 5.2, 0.001);
      assertEquals(oSubs.m_fInitCondScarSoil.GetValue(), 0.23, 0.001);
      assertEquals(oSubs.m_fPartialCutScarSoil.GetValue(), 0.17, 0.001);
      assertEquals(oSubs.m_fGapCutScarSoil.GetValue(), 0.45, 0.001);
      assertEquals(oSubs.m_fClearCutScarSoil.GetValue(), 0.36, 0.001);
      assertEquals(oSubs.m_fInitCondTipup.GetValue(), 0.2, 0.001);
      assertEquals(oSubs.m_fPartialCutTipup.GetValue(), 0.15, 0.001);
      assertEquals(oSubs.m_fGapCutTipup.GetValue(), 0.34, 0.001);
      assertEquals(oSubs.m_fClearCutTipup.GetValue(), 0.32, 0.001);
      assertEquals(oSubs.m_fInitCondFreshLog.GetValue(), 0.01, 0.001);
      assertEquals(oSubs.m_fPartialCutFreshLog.GetValue(), 0.06, 0.001);
      assertEquals(oSubs.m_fGapCutFreshLog.GetValue(), 0.09, 0.001);
      assertEquals(oSubs.m_fClearCutFreshLog.GetValue(), 0.13, 0.001);
      assertEquals(oSubs.m_fInitCondDecLog.GetValue(), 0.11, 0.001);
      assertEquals(oSubs.m_fPartialCutDecLog.GetValue(), 0.04, 0.001);
      assertEquals(oSubs.m_fGapCutDecLog.GetValue(), 0.12, 0.001);
      assertEquals(oSubs.m_fClearCutDecLog.GetValue(), 0.02, 0.001);
      assertEquals(oSubs.m_fRootTipupFactor.GetValue(), 3.1, 0.001);
      assertEquals(oSubs.m_fMossProportion.GetValue(), 0.7, 0.001);
      assertEquals(oSubs.m_iDirectionalTreeFall.GetValue(), 1);
      assertEquals(oSubs.m_iMaxDecayTime.GetValue(), 3);
      assertEquals( ( (Float) oSubs.mp_fProportionOfDeadThatFall.GetValue().elementAt(0)).floatValue(), 1, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfDeadThatFall.GetValue().elementAt(1)).floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().elementAt(1)).floatValue(), 0.18, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().elementAt(0)).floatValue(), 0.4, 0.001);
      assertEquals( ( (Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().elementAt(1)).floatValue(), 0.8, 0.001);
    }
    catch (ModelException oErr) {
      fail("Substrate backwards compatibility failed with message " +
           oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      if (sFileName != null) {
        new File(sFileName).delete();
      }
    }
  }

  /**
   * Tests ValidateData().
   * @throws ModelException if there's an uncaught validation problem.
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    SubstrateBehaviors oSubs = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oSubs = oManager.GetSubstrateBehaviors();
      oSubs.ValidateData(oManager.GetTreePopulation());
    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Case:  root tip-up factor is negative
    try {
      oSubs.m_fRootTipupFactor.SetValue(-3);
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch negative root tipup factor.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    oSubs.m_fRootTipupFactor.SetValue(3);
    oSubs.ValidateData(oManager.GetTreePopulation());

    //Case: max decay time is negative
    try {
      oSubs.m_iMaxDecayTime.SetValue(-3);
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch negative max decay time.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    oSubs.m_iMaxDecayTime.SetValue(3);
    oSubs.ValidateData(oManager.GetTreePopulation());

    //Case:  proportion of dead that fall is not a proportion
    try {
      oSubs.mp_fProportionOfDeadThatFall.GetValue().clear();
      oSubs.mp_fProportionOfDeadThatFall.GetValue().add(new Float(0.2));
      oSubs.mp_fProportionOfDeadThatFall.GetValue().add(new Float(1.2));
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch bad value for proportion of dead that fall.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    try {
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oSubs = oManager.GetSubstrateBehaviors();
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Case:  proportion of fallen that uproot is not a proportion
    try {
      oSubs.mp_fProportionOfFallenThatUproot.GetValue().clear();
      oSubs.mp_fProportionOfFallenThatUproot.GetValue().add(new Float(0.2));
      oSubs.mp_fProportionOfFallenThatUproot.GetValue().add(new Float(-1.2));
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch bad value for proportion of fallen that uproot.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    try {
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oSubs = oManager.GetSubstrateBehaviors();
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Case:  proportion of snags that uproot is not a proportion
    try {
      oSubs.mp_fProportionOfSnagsThatUproot.GetValue().clear();
      oSubs.mp_fProportionOfSnagsThatUproot.GetValue().add(new Float(0.2));
      oSubs.mp_fProportionOfSnagsThatUproot.GetValue().add(new Float(-1.2));
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch bad value for proportion of snags that uproot.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    try {
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oSubs = oManager.GetSubstrateBehaviors();
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Case:  proportion of moss isn't a proportion
    try {
      oSubs.m_fMossProportion.SetValue(-3);
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch bad moss proportion value.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    oSubs.m_fMossProportion.SetValue((float)0.3);
    oSubs.ValidateData(oManager.GetTreePopulation());

    //Case:  initial conditions add up to greater than 1
    try {
      oSubs.m_fInitCondDecLog.SetValue((float)0.9);
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch initial conditions greater than 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    oSubs.m_fInitCondDecLog.SetValue((float)0.01);
    oSubs.ValidateData(oManager.GetTreePopulation());

    //Case:  partial cut harvest conditions add up to greater than 1
    try {
      oSubs.m_fPartialCutDecLog.SetValue((float)0.9);
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch partial cut conditions greater than 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    oSubs.m_fPartialCutDecLog.SetValue((float)0.01);
    oSubs.ValidateData(oManager.GetTreePopulation());

    //Case:  gap cut harvest conditions add up to greater than 1
    try {
      oSubs.m_fGapCutDecLog.SetValue((float)0.9);
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch gap cut conditions greater than 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    oSubs.m_fGapCutDecLog.SetValue((float)0.01);
    oSubs.ValidateData(oManager.GetTreePopulation());

    //Case:  clear cut harvest conditions add up to greater than 1
    try {
      oSubs.m_fClearCutDecLog.SetValue((float)0.9);
      oSubs.ValidateData(oManager.GetTreePopulation());
      fail("Substrate didn't catch clear cut conditions greater than 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    oSubs.m_fClearCutDecLog.SetValue((float)0.01);
    oSubs.ValidateData(oManager.GetTreePopulation());

    System.out.println("Substrate ValidateData testing succeeded.");
  }

  /**
   * Writes an old-school substrate file to test backwards compatibility.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>June 3, 2005: Created (LEM)
   */
  protected String WriteXMLOldFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>substrate</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<substrate>");
    oOut.write("<su_gapmossDecayAlpha>-5.1E-4</su_gapmossDecayAlpha>");
    oOut.write("<su_gapmossDecayBeta>4.4</su_gapmossDecayBeta>");
    oOut.write("<su_organicDecayAlpha>-7.0E-4</su_organicDecayAlpha>");
    oOut.write("<su_organicDecayBeta>4.3</su_organicDecayBeta>");
    oOut.write("<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>");
    oOut.write("<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>");
    oOut.write("<su_mossyLogToCanMoss>0.55</su_mossyLogToCanMoss>");
    oOut.write("<su_maxNumberDecayYears>50</su_maxNumberDecayYears>");
    oOut.write("<su_initialGapMoss>0.18</su_initialGapMoss>");
    oOut.write("<su_initialOrganic>0.3</su_initialOrganic>");
    oOut.write("<su_initialFreshLog>0.01</su_initialFreshLog>");
    oOut.write("<su_initialMossyLog>0.11</su_initialMossyLog>");
    oOut.write("<su_partialCutGapMoss>0.17</su_partialCutGapMoss>");
    oOut.write("<su_partialCutOrganic>0.15</su_partialCutOrganic>");
    oOut.write("<su_partialCutFreshLog>0.06</su_partialCutFreshLog>");
    oOut.write("<su_partialCutMossyLog>0.04</su_partialCutMossyLog>");
    oOut.write("<su_gapCutGapMoss>0.45</su_gapCutGapMoss>");
    oOut.write("<su_gapCutOrganic>0.34</su_gapCutOrganic>");
    oOut.write("<su_gapCutFreshLog>0.09</su_gapCutFreshLog>");
    oOut.write("<su_gapCutMossyLog>0.12</su_gapCutMossyLog>");
    oOut.write("<su_clearCutGapMoss>0.36</su_clearCutGapMoss>");
    oOut.write("<su_clearCutOrganic>0.35</su_clearCutOrganic>");
    oOut.write("<su_clearCutFreshLog>0.16</su_clearCutFreshLog>");
    oOut.write("<su_clearCutMossyLog>0.01</su_clearCutMossyLog>");
    oOut.write("<su_propOfDeadFall>");
    oOut.write("<su_podfVal species=\"Species_1\">0.7</su_podfVal>");
    oOut.write("<su_podfVal species=\"Species_2\">0.6</su_podfVal>");
    oOut.write("</su_propOfDeadFall>");
    oOut.write("<su_propOfFallUproot>");
    oOut.write("<su_pofuVal species=\"Species_1\">0.5</su_pofuVal>");
    oOut.write("<su_pofuVal species=\"Species_2\">0.4</su_pofuVal>");
    oOut.write("</su_propOfFallUproot>");
    oOut.write("<su_propOfSnagsUproot>");
    oOut.write("<su_posuVal species=\"Species_1\">0.3</su_posuVal>");
    oOut.write("<su_posuVal species=\"Species_2\">0.2</su_posuVal>");
    oOut.write("</su_propOfSnagsUproot>");
    oOut.write("<su_rootTipupFactor>3.1</su_rootTipupFactor>");
    oOut.write("</substrate>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a 6.05 substrate file.
   * @throws IOException If there is a problem writing the file.
   * @return String Filename of file written.
   */
  protected String WriteXMLOldFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>substrate</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<substrate>");
    oOut.write("<su_scarSoilDecayAlpha>-5.1</su_scarSoilDecayAlpha>");
    oOut.write("<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>");
    oOut.write("<su_tipupDecayAlpha>-7.0</su_tipupDecayAlpha>");
    oOut.write("<su_tipupDecayBeta>4.3</su_tipupDecayBeta>");
    oOut.write("<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>");
    oOut.write("<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>");
    oOut.write("<su_DecayedLogToForestFloor>0.75</su_DecayedLogToForestFloor>");
    oOut.write("<su_maxNumberDecayYears>3</su_maxNumberDecayYears>");
    oOut.write("<su_initialScarSoil>0.23</su_initialScarSoil>");
    oOut.write("<su_initialTipup>0.2</su_initialTipup>");
    oOut.write("<su_initialFreshLog>0.01</su_initialFreshLog>");
    oOut.write("<su_initialDecayedLog>0.11</su_initialDecayedLog>");
    oOut.write("<su_partialCutScarSoil>0.17</su_partialCutScarSoil>");
    oOut.write("<su_partialCutTipup>0.15</su_partialCutTipup>");
    oOut.write("<su_partialCutFreshLog>0.06</su_partialCutFreshLog>");
    oOut.write("<su_partialCutDecayedLog>0.04</su_partialCutDecayedLog>");
    oOut.write("<su_gapCutScarSoil>0.45</su_gapCutScarSoil>");
    oOut.write("<su_gapCutTipup>0.34</su_gapCutTipup>");
    oOut.write("<su_gapCutFreshLog>0.09</su_gapCutFreshLog>");
    oOut.write("<su_gapCutDecayedLog>0.12</su_gapCutDecayedLog>");
    oOut.write("<su_clearCutScarSoil>0.36</su_clearCutScarSoil>");
    oOut.write("<su_clearCutTipup>0.32</su_clearCutTipup>");
    oOut.write("<su_clearCutFreshLog>0.13</su_clearCutFreshLog>");
    oOut.write("<su_clearCutDecayedLog>0.02</su_clearCutDecayedLog>");
    oOut.write("<su_propOfDeadFall>");
    oOut.write("<su_podfVal species=\"Species_1\">1</su_podfVal>");
    oOut.write("<su_podfVal species=\"Species_2\">0.2</su_podfVal>");
    oOut.write("</su_propOfDeadFall>");
    oOut.write("<su_propOfFallUproot>");
    oOut.write("<su_pofuVal species=\"Species_1\">0</su_pofuVal>");
    oOut.write("<su_pofuVal species=\"Species_2\">0.18</su_pofuVal>");
    oOut.write("</su_propOfFallUproot>");
    oOut.write("<su_propOfSnagsUproot>");
    oOut.write("<su_posuVal species=\"Species_1\">0.4</su_posuVal>");
    oOut.write("<su_posuVal species=\"Species_2\">0.8</su_posuVal>");
    oOut.write("</su_propOfSnagsUproot>");
    oOut.write("<su_mossProportion>0.7</su_mossProportion>");
    oOut.write("<su_directionalTreeFall>1</su_directionalTreeFall>");
    oOut.write("<su_rootTipupFactor>3.1</su_rootTipupFactor>");
    oOut.write("</substrate>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a 6.05 substrate file.
   * @throws IOException If there is a problem writing the file.
   * @return String Filename of file written.
   */
  protected String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>substrate</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<substrate>");
    oOut.write("<su_scarSoilDecayAlpha>-5.1</su_scarSoilDecayAlpha>");
    oOut.write("<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>");
    oOut.write("<su_tipupDecayAlpha>-7.0</su_tipupDecayAlpha>");
    oOut.write("<su_tipupDecayBeta>4.3</su_tipupDecayBeta>");
    oOut.write("<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>");
    oOut.write("<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>");
    oOut.write("<su_decayedLogDecayAlpha>-0.2</su_decayedLogDecayAlpha>");
    oOut.write("<su_decayedLogDecayBeta>5.2</su_decayedLogDecayBeta>");
    oOut.write("<su_maxNumberDecayYears>3</su_maxNumberDecayYears>");
    oOut.write("<su_initialScarSoil>0.23</su_initialScarSoil>");
    oOut.write("<su_initialTipup>0.2</su_initialTipup>");
    oOut.write("<su_initialFreshLog>0.01</su_initialFreshLog>");
    oOut.write("<su_initialDecayedLog>0.11</su_initialDecayedLog>");
    oOut.write("<su_partialCutScarSoil>0.17</su_partialCutScarSoil>");
    oOut.write("<su_partialCutTipup>0.15</su_partialCutTipup>");
    oOut.write("<su_partialCutFreshLog>0.06</su_partialCutFreshLog>");
    oOut.write("<su_partialCutDecayedLog>0.04</su_partialCutDecayedLog>");
    oOut.write("<su_gapCutScarSoil>0.45</su_gapCutScarSoil>");
    oOut.write("<su_gapCutTipup>0.34</su_gapCutTipup>");
    oOut.write("<su_gapCutFreshLog>0.09</su_gapCutFreshLog>");
    oOut.write("<su_gapCutDecayedLog>0.12</su_gapCutDecayedLog>");
    oOut.write("<su_clearCutScarSoil>0.36</su_clearCutScarSoil>");
    oOut.write("<su_clearCutTipup>0.32</su_clearCutTipup>");
    oOut.write("<su_clearCutFreshLog>0.13</su_clearCutFreshLog>");
    oOut.write("<su_clearCutDecayedLog>0.02</su_clearCutDecayedLog>");
    oOut.write("<su_propOfDeadFall>");
    oOut.write("<su_podfVal species=\"Species_1\">1</su_podfVal>");
    oOut.write("<su_podfVal species=\"Species_2\">0.2</su_podfVal>");
    oOut.write("</su_propOfDeadFall>");
    oOut.write("<su_propOfFallUproot>");
    oOut.write("<su_pofuVal species=\"Species_1\">0</su_pofuVal>");
    oOut.write("<su_pofuVal species=\"Species_2\">0.18</su_pofuVal>");
    oOut.write("</su_propOfFallUproot>");
    oOut.write("<su_propOfSnagsUproot>");
    oOut.write("<su_posuVal species=\"Species_1\">0.4</su_posuVal>");
    oOut.write("<su_posuVal species=\"Species_2\">0.8</su_posuVal>");
    oOut.write("</su_propOfSnagsUproot>");
    oOut.write("<su_mossProportion>0.7</su_mossProportion>");
    oOut.write("<su_directionalTreeFall>1</su_directionalTreeFall>");
    oOut.write("<su_rootTipupFactor>3.1</su_rootTipupFactor>");
    oOut.write("</substrate>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * This writes plot and tree information.
   * @param oOut FileWriter
   * @throws IOException
   */
  private void WritePlotAndTrees(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

}
